"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const extract_pg_schema_1 = require("extract-pg-schema");
const rmfr_1 = __importDefault(require("rmfr"));
const default_metadata_generators_1 = require("./default-metadata-generators");
const defaultTypeMap_1 = __importDefault(require("./defaultTypeMap"));
const makeCompositeGenerator_1 = __importDefault(require("./generators/makeCompositeGenerator"));
const makeDomainsGenerator_1 = __importDefault(require("./generators/makeDomainsGenerator"));
const makeEnumsGenerator_1 = __importDefault(require("./generators/makeEnumsGenerator"));
const makeRangesGenerator_1 = __importDefault(require("./generators/makeRangesGenerator"));
const markAsGenerated_1 = __importDefault(require("./hooks/markAsGenerated"));
const render_1 = __importDefault(require("./render"));
const writeFile_1 = __importDefault(require("./writeFile"));
const processDatabase = async (config, progress) => {
    const schemas = await (0, extract_pg_schema_1.extractSchemas)(config.connection, {
        schemas: config.schemas,
        typeFilter: config.typeFilter,
        ...progress,
    });
    const typeMap = {
        ...defaultTypeMap_1.default,
        ...config.customTypeMap,
    };
    const getMetadata = config.getMetadata ?? default_metadata_generators_1.defaultGetMetadata;
    const getPropertyMetadata = config.getPropertyMetadata ?? default_metadata_generators_1.defaultGetPropertyMetadata;
    const generateIdentifierType = config.generateIdentifierType ?? default_metadata_generators_1.defaultGenerateIdentifierType;
    const propertySortFunction = config.propertySortFunction ?? default_metadata_generators_1.defaultPropertySortFunction;
    const instantiatedConfig = {
        getMetadata,
        getPropertyMetadata,
        generateIdentifierType,
        propertySortFunction,
        enumStyle: config.enumStyle ?? "enum",
        typeMap,
        schemas,
        connection: config.connection,
        outputPath: config.outputPath ?? ".",
        preDeleteOutputFolder: config.preDeleteOutputFolder ?? false,
        resolveViews: config.resolveViews ?? true,
    };
    const generators = [
        (0, makeCompositeGenerator_1.default)("table", instantiatedConfig),
        (0, makeCompositeGenerator_1.default)("view", instantiatedConfig),
        (0, makeCompositeGenerator_1.default)("materializedView", instantiatedConfig),
        (0, makeCompositeGenerator_1.default)("compositeType", instantiatedConfig),
        (0, makeEnumsGenerator_1.default)(instantiatedConfig),
        (0, makeRangesGenerator_1.default)(instantiatedConfig),
        (0, makeDomainsGenerator_1.default)(instantiatedConfig),
    ];
    let output = {};
    Object.values(schemas).forEach((schema) => {
        generators.forEach((generator) => {
            output = generator(schema, output);
        });
    });
    const preRenderHooks = config.preRenderHooks ?? [];
    for (const hook of preRenderHooks) {
        output = await hook(output, instantiatedConfig);
    }
    let filesToWrite = Object.keys(output).map((path) => {
        const lines = (0, render_1.default)(output[path].declarations, path);
        return { fullPath: `${path}.ts`, lines };
    });
    const postRenderHooks = config.postRenderHooks ?? [markAsGenerated_1.default];
    for (const hook of postRenderHooks) {
        filesToWrite = await Promise.all(filesToWrite.map(async (file) => {
            const lines = await hook(file.fullPath, file.lines, instantiatedConfig);
            return { ...file, lines };
        }));
    }
    if (instantiatedConfig.preDeleteOutputFolder) {
        console.info(`Clearing old files in ${instantiatedConfig.outputPath}`);
        await (0, rmfr_1.default)(instantiatedConfig.outputPath, { glob: true });
    }
    filesToWrite.forEach((file) => (0, writeFile_1.default)(file));
};
exports.default = processDatabase;
