"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const render_1 = __importDefault(require("./render"));
(0, vitest_1.describe)("processGenerationSetup", () => {
    (0, vitest_1.it)("should process a type declaration", () => {
        const declarations = [
            {
                declarationType: "typeDeclaration",
                name: "MyString",
                exportAs: "named",
                comment: ["This is just a string"],
                typeDefinition: ["string"],
            },
        ];
        const lines = (0, render_1.default)(declarations, "./");
        (0, vitest_1.expect)(lines).toEqual([
            "/** This is just a string */",
            `export type MyString = string;`,
        ]);
    });
    (0, vitest_1.it)("should work with multi-line type definitions", () => {
        const declarations = [
            {
                declarationType: "typeDeclaration",
                name: "MyUnion",
                exportAs: "named",
                comment: [],
                typeDefinition: ["", "| 'apples'", "| 'oranges'", "| 'bananas'"],
            },
        ];
        const lines = (0, render_1.default)(declarations, "./");
        (0, vitest_1.expect)(lines).toEqual([
            `export type MyUnion = `,
            `  | 'apples'`,
            `  | 'oranges'`,
            `  | 'bananas';`,
        ]);
    });
    (0, vitest_1.it)("should process an interface", () => {
        const declarations = [
            {
                declarationType: "interface",
                name: "Member",
                exportAs: "default",
                properties: [
                    {
                        name: "id",
                        typeName: "MemberId",
                        comment: ["This is the identifier", "it's a number"],
                        dimensions: 0,
                        isOptional: false,
                        isNullable: false,
                    },
                ],
            },
        ];
        const lines = (0, render_1.default)(declarations, "./");
        (0, vitest_1.expect)(lines).toEqual([
            `export default interface Member {`,
            "  /**",
            "   * This is the identifier",
            "   * it's a number",
            "   */",
            "  id: MemberId;",
            "}",
        ]);
    });
    (0, vitest_1.it)("should process an enum", () => {
        const declarations = [
            {
                declarationType: "enum",
                name: "MpaaRating",
                exportAs: "named",
                values: ["G", "PG", "PG-13", "R", "NC-17"],
            },
        ];
        const lines = (0, render_1.default)(declarations, "./");
        (0, vitest_1.expect)(lines).toEqual([
            `export enum MpaaRating {`,
            `  G = 'G',`,
            `  PG = 'PG',`,
            `  'PG-13' = 'PG-13',`,
            `  R = 'R',`,
            `  'NC-17' = 'NC-17',`,
            `};`,
        ]);
    });
    (0, vitest_1.it)("should support a default exported enum", () => {
        const declarations = [
            {
                declarationType: "enum",
                name: "Fruit",
                exportAs: "default",
                values: ["apples", "oranges", "bananas"],
            },
        ];
        const lines = (0, render_1.default)(declarations, "./");
        (0, vitest_1.expect)(lines).toEqual([
            `enum Fruit {`,
            `  apples = 'apples',`,
            `  oranges = 'oranges',`,
            `  bananas = 'bananas',`,
            `};`,
            "",
            "export default Fruit;",
        ]);
    });
    (0, vitest_1.it)("should process a constant", () => {
        const declarations = [
            {
                declarationType: "constant",
                name: "MyConstant",
                type: "boolean",
                exportAs: "named",
                value: "true",
            },
        ];
        const lines = (0, render_1.default)(declarations, "./");
        (0, vitest_1.expect)(lines).toEqual([`export const MyConstant: boolean = true;`]);
    });
    (0, vitest_1.it)("should process a multi-line constant", () => {
        const declarations = [
            {
                declarationType: "constant",
                name: "actor",
                type: "z.Schema<Actor>",
                exportAs: "named",
                value: ["z.object({", "  actor_id: actorId,", "})"],
            },
        ];
        const lines = (0, render_1.default)(declarations, "./");
        (0, vitest_1.expect)(lines).toEqual([
            "export const actor: z.Schema<Actor> = z.object({",
            `  actor_id: actorId,`,
            `});`,
        ]);
    });
});
